## Function to simulate snp data

## INPUTS

## X    : matrix of subject by snp
## case : number of cases to simulate
## ctl  : numer of controls to simulate
## b0   : model intercept 
## b    : regression coefficients, weights to be applied to snps

## OUTPUTS

# The single output is a dataframe of subject by snps. The first 
# column is the outcome "Y" indicating case (disease, Y=1) or 
# control (no disease, Y=0).
# NOTE : The number of snps output is the same as the number of
# snps input. The user only has control over the number of cases
# and controls to simulate. 
 
simulation <- function(X, case, ctl, b0, b, seed = 1263940)
{
    set.seed(seed)
    snpNames <- colnames(X)
    colnames(X) <- rownames(X) <- NULL

    ## Compute p and q of logistic model
    z <- b0 + X%*%b
    p <- 1 - 1/(1 + exp(z))
    q <- 1 - p

    ## Normalize p and q by computing proportions and cumsums
    df <- data.frame(order=seq_len(length(p)), p=p, q=q)
    df[["p_prop"]] <- sapply(df[["p"]], function(x) x/sum(df[["p"]]))
    df[["q_prop"]] <- sapply(df[["q"]], function(x) x/sum(df[["q"]]))
    dfsortp <- df[order(df[["p_prop"]]), ]
    dfsortp[["p_cumsum"]] <- cumsum(dfsortp[["p_prop"]])
    dfsortq <- dfsortp[order(dfsortp[["q_prop"]]), ]
    dfsortq[["q_cumsum"]] <- cumsum(dfsortq[["q_prop"]])
    dfsort <- dfsortq

    ## Generate cases (Y = 1) :
    casegen <- data.frame(index=c(rep(NA, case), dfsort[["order"]]),
                          values=c(runif(case), dfsort[["p_cumsum"]]),
                          flag=c(rep.int(1, case), 
                              rep.int(0, length(dfsort[["p_cumsum"]]))))
 
    casegen <- casegen[order(casegen[["values"]]), ]
    casegen[["sum"]] <- cumsum(casegen[["flag"]])
 
    casesub <- casegen[casegen[["flag"]] == 0, ]
    casesub[["diff"]] <- diff(c(0, casesub[["sum"]]))
 
    ycase <- rep.int(1, case)
    Xcase <- apply(X[casesub[["index"]],], 2, 
                   function(i) rep(i, casesub[["diff"]]))
 
    ## Generate controls (Y = 0) :
    ctlgen <- data.frame(index=c(rep(NA, ctl), dfsort[["order"]]),
                         values=c(runif(ctl), dfsort[["q_cumsum"]]),
                         flag=c(rep.int(1, ctl),
                             rep.int(0, length(dfsort[["q_cumsum"]]))))

    ctlgen <- ctlgen[order(ctlgen[["values"]]), ]
    ctlgen[["sum"]] <- cumsum(ctlgen[["flag"]])

    ctlsub <- ctlgen[ctlgen[["flag"]] == 0, ]
    ctlsub[["diff"]] <- diff(c(0, ctlsub[["sum"]]))

    yctl <- rep.int(0, ctl)
    Xctl <- apply(X[ctlsub[["index"]],], 2,
                   function(i) rep(i, ctlsub[["diff"]]))
 
    finalData <- data.frame(Y=c(ycase, yctl), x=rbind(Xcase, Xctl))

    colnames(finalData) <- c("Y", snpNames) 
    finalData
}

